@@ -40,22 +40,25 @@
 #' 
 #' These tests are originally meant to use the residuals of separate
 #' estimation of one time--series regression for each cross-sectional
-#' unit in order to check for cross--sectional dependence. If a
-#' different model specification (`within`, `random`, \ldots{}) is
-#' assumed consistent, one can resort to its residuals for testing
-#' (which is common, e.g., when the time dimension's length is
-#' insufficient for estimating the heterogeneous model).  If the time
-#' dimension is insufficient and `model=NULL`, the function defaults
+#' unit in order to check for cross--sectional dependence (`model = NULL`).
+#' If a different model specification (`model = "within"`, `"random"`, 
+#' \ldots{}) is assumed consistent, one can resort to its residuals for
+#' testing (which is common, e.g., when the time dimension's length is
+#' insufficient for estimating the heterogeneous model).
+#' 
+#' If the time
+#' dimension is insufficient and `model = NULL`, the function defaults
 #' to estimation of a `within` model and issues a warning. The main
 #' argument of this function may be either a model of class
-#' `panelmodel` or a `formula` and `dataframe`; in the second case,
+#' `panelmodel` or a `formula` and `data frame`; in the second case,
 #' unless `model` is set to `NULL`, all usual parameters relative to
 #' the estimation of a `plm` model may be passed on. The test is
 #' compatible with any consistent `panelmodel` for the data at hand,
-#' with any specification of `effect`. E.g., specifying
-#' `effect="time"` or `effect="twoways"` allows to test for residual
-#' cross-sectional dependence after the introduction of time fixed
-#' effects to account for common shocks.
+#' with any specification of `effect` (except for `test = "bcsclm"` which
+#' requires a within model with either individual or two-ways effect).
+#' E.g., specifying  `effect = "time"` or `effect = "twoways"` allows
+#' to test for residual cross-sectional dependence after the introduction
+#' of time fixed effects to account for common shocks.
 #' 
 #' A **local** version of either test can be computed by supplying a
 #' proximity matrix (elements coercible to `logical`) with argument
@@ -117,8 +120,8 @@
 #'     `as.logical()` is evaluated for neighbouring information (but
 #'     `w` can be symmetric). See also **Details** and
 #'     **Examples**,
-#' @param \dots further arguments to be passed on to `plm`, such as
-#'     `effect` or `random.method`.
+#' @param \dots further arguments to be passed on for model estimation to `plm`,
+#'    such as `effect` or `random.method`.
 #' @return An object of class `"htest"`.
 #' @export
 #' @references
@@ -175,53 +178,82 @@ pcdtest.formula <- function(x, data, index = NULL, model = NULL,
                             w = NULL, ...) {
     #data <- pdata.frame(data, index = index)
     test <- match.arg(test)
-    if (is.null(model) && test == "bcsclm") stop("for test = 'bcsclm', set argument model = 'within'")
-    mymod <- if (test != "bcsclm") plm(x, data = data, index = index, model = "pooling", ...)
-              else plm(x, data = data, index = index, model = "within", ...)
-    if(is.null(model) && min(pdim(mymod)$Tint$Ti) < length(mymod$coefficients)+1) 
-      {
-        warning("Insufficient number of observations in time to estimate heterogeneous model: using within residuals",
-            call. = FALSE)
-        model <- "within"
-    }
+    if(test == "bcsclm" && (is.null(model) || model != "within"))
+      stop("for test = 'bcsclm', set argument model = 'within'")
+
+    # evaluate formula in parent frame
+    cl <- match.call(expand.dots = TRUE)
+    cl$model  <- if(test != "bcsclm") "pooling" else "within"
+      if(test == "bcsclm") {
+        # check args model and effect for test = "bcsclm"
+        if(is.null(cl$effect)) cl$effect <- "individual" # make default within model is individual within
+        eff <- isTRUE(cl$effect == "individual" || cl$effect == "twoways")
+        if(model != "within" || !eff) stop("for test = 'bcsclm', requirement is model = \"within\" and effect = \"individual\" or \"twoways\"")
+      }
+    names(cl)[2L] <- "formula"
+    m <- match(plm.arg, names(cl), 0L)
+    cl <- cl[c(1L, m)]
+    cl[[1L]] <- as.name("plm")
+    mymod <- eval(cl, parent.frame()) # mymod is either "pooling" or "within" (the latter iff for test = "bcsclm")
     
-    ind0 <- attr(model.frame(mymod), "index")
-    tind <- as.numeric(ind0[[2]])
-    ind <- as.numeric(ind0[[1]])
-    if (is.null(model)) {
-        ## estimate individual regressions one by one
+    hetero.spec <- if(is.null(model)) TRUE else FALSE
+    
+    if(hetero.spec && min(pdim(mymod)$Tint$Ti) < length(mymod$coefficients)+1) {
+      warning("Insufficient number of observations in time to estimate heterogeneous model: using within residuals",
+            call. = FALSE)
+      hetero.spec <- FALSE
+      model <- "within"
+    }
+
+    ind0 <- unclass(attr(model.frame(mymod), "index")) # unclass for speed
+    tind <- as.numeric(ind0[[2L]])
+    ind  <- as.numeric(ind0[[1L]])
+
+    unind <- unique(ind)
+    n <- length(unind)
+    
+    if(hetero.spec) {
+        ## estimate individual normal regressions one by one
+        ## (original heterogeneous specification of Pesaran)
         X <- model.matrix(mymod)
         y <- model.response(model.frame(mymod))
-        unind <- unique(ind)
-        n <- length(unind)
-        ti.res <- vector("list", n)
-        ind.res <- vector("list", n)
-        tind.res <- vector("list", n)
-        for (i in 1:n) {
-            tX <- X[ind == unind[i], , drop = FALSE]
-            ty <- y[ind == unind[i]]
-            res.i <- lm.fit(tX, ty)$residuals
-            ti.res[[i]] <- res.i
-            names(ti.res[[i]]) <- tind[ind == unind[i]]
-            ind.res[[i]] <- rep(i, length(res.i))
-            tind.res[[i]] <- tind[ind == unind[i]]
-        }
+ 
+        # split X, y per individual
+        ind.GRP <- collapse::GRP(ind)
+        tX <- collapse::rsplit(X, ind.GRP, use.names = FALSE)
+        ty <- collapse::gsplit(y, ind.GRP)
+
+        # calc. residuals
+        res.i <- mapply(function(X, y) lm.fit(X, y)$residuals, tX, ty, SIMPLIFY = FALSE)
+        
+        # construct indexes
+        ind.i <- rep(seq_len(n), lengths(res.i))
+        tind.i <- collapse::gsplit(tind, ind.GRP)
+        tind.i <- unlist(tind.i, use.names = FALSE)
+        
         ## make pseries of (all) residuals
-        resdata <- data.frame(ee = unlist(ti.res),
-                              ind = unlist(ind.res),
-                              tind = unlist(tind.res))
+        resdata <- data.frame(ee   = unlist(res.i,  use.names = FALSE),
+                              ind  = ind.i,
+                              tind = tind.i)
+        
         pee <- pdata.frame(resdata, index = c("ind", "tind"))
         tres <- pee$ee
+    } else {
+      # else case is one of:
+      # a) insufficient number of observations for heterogen. spec. or
+      # b) model specified when function was called (incl. case test = "bcsclm")
+      if(test != "bcsclm") {
+        # Estimate the model specified originally in function call or due to
+        # forced model switch to within model by insufficient number of
+        # observations for heterogen. spec.
+        # (for test = "bcsclm" it is ensured that a within model was already
+        # estimated -> no need to estimate again a within model)
+        cl$model <- model
+        mymod <- eval(cl, parent.frame())
+      }
+      
+      tres <- resid(mymod)
     }
-    else {
-        mymod <- plm(x, data, index = index, model = model, ...)
-        tres <- resid(mymod)
-        unind <- unique(ind)
-        n <- length(unind)
-        t <- min(pdim(mymod)$Tint$Ti)
-        nT <- length(ind)
-        k <- length(mymod$coefficients)
-        }
 
     return(pcdres(tres = tres, n = n, w = w,
                   form = paste(deparse(x)),
@@ -240,13 +272,13 @@ pcdtest.panelmodel <- function(x, test = c("cd", "sclm", "bcsclm", "lm", "rho",
     model <- describe(x, "model")
     effect <- describe(x, "effect")
     eff <- (effect == "individual" || effect == "twoways")
-    if (test == "bcsclm")
-      if (model != "within" || !eff) stop("for test = 'bcsclm', model x must be a within individual or twoways model")
-  
+    if (test == "bcsclm" && (model != "within" || !eff))
+      stop("for test = 'bcsclm', model x must be a within individual or twoways model")
+    
     tres <- resid(x)
-    index <- attr(model.frame(x), "index")
-    #tind <- as.numeric(index[[2]])
-    ind <- as.numeric(index[[1]])
+    index <- unclass(attr(model.frame(x), "index")) # unclass for speed
+    #tind <- as.numeric(index[[2L]])
+    ind <- as.numeric(index[[1L]])
     unind <- unique(ind)
     n <- length(unind)
     #t <- pdim(x)$Tint$Ti
@@ -272,36 +304,21 @@ pcdtest.pseries <- function(x, test = c("cd", "sclm", "bcsclm", "lm", "rho", "ab
   
     pos.na <- is.na(x)
     if (any(pos.na)) {
-      x <- subset_pseries(x, !pos.na)
+      x <- subset_pseries(x, !pos.na) # TODO: use [.pseries (pseries subsetting) once implemented
       warning("NA values encountered in input and removed")
       if (length(x) == 0L) stop("input is empty after removal of NA values")
     }
   
     ## get indices
-    tind <- as.numeric(attr(x, "index")[[2]])
-    ind <- as.numeric(attr(x, "index")[[1]])
+    ix <- unclass(attr(x, "index")) # unclass for speed
+    tind <- as.numeric(ix[[2L]])
+    ind  <- as.numeric(ix[[1L]])
 
     ## det. number of groups and df
     unind <- unique(ind)
     n <- length(unind)
 
-    tres <- x
-    
-    ## "pre-allocate" an empty list of length n
-    #tres <- vector("list", n)
-
-    ## use model residuals, group by group
-    ## list of n:
-    ## t_i residuals for each x-sect. 1..n
-    #for(i in 1:n) {
-    #          # remove NAs
-    #          xnonna <- !is.na(x[ind==unind[i]])
-    #          tres[[i]] <- x[ind==unind[i]][xnonna]
-    #          ## name resids after the time index
-    #          names(tres[[i]]) <- tind[ind==unind[i]][xnonna]
-    #          }
-
-    return(pcdres(tres = tres, n = n, w = w,
+    return(pcdres(tres = x, n = n, w = w,
                   form = form,
                   test = match.arg(test)))
 }
@@ -326,19 +343,18 @@ pcdres <- function(tres, n, w, form, test) {
     
     ## find length of intersecting pairs
     ## fast method, times down 200x
-    data.res <- data.frame(time = attr(tres, "index")[[2]],
-                           indiv = attr(tres, "index")[[1]])
+    ix <- unclass(attr(tres, "index")) # unclass for speed
     ## tabulate which obs in time for each ind are !na
-    presence.tab <- table(data.res)
+    presence.tab <- collapse::qtable(ix[[2L]], ix[[1L]])
     ## calculate t.ij
     t.ij <- crossprod(presence.tab)
     
   # input check
   if (!is.null(w)) {
     dims.w <- dim(w)
-    if(dims.w[1] != n || dims.w[2] != n)
+    if(dims.w[1L] != n || dims.w[2L] != n)
       stop(paste0("matrix 'w' describing proximity of individuals has wrong dimensions: ",
-           "should be ", n, " x ", n, " (no. of individuals) but is ", dims.w[1], " x ", dims.w[2]))
+           "should be ", n, " x ", n, " (no. of individuals) but is ", dims.w[1L], " x ", dims.w[2L]))
   }
   
 
@@ -359,7 +375,7 @@ pcdres <- function(tres, n, w, form, test) {
   ## about row-std. matrices)
   selector.mat <- matrix(as.logical(w), ncol = n)
   
-  ## some santiy checks for 'w' (not perfect sanity, but helps)
+  ## some sanity checks for 'w' (not perfect sanity, but helps)
   if (sum(selector.mat[lower.tri(selector.mat, diag = FALSE)]) == 0) {
     stop(paste0("no neighbouring individuals defined in proximity matrix 'w'; ",
                 "only lower triangular part of 'w' (w/o diagonal) is evaluated"))
@@ -375,7 +391,7 @@ pcdres <- function(tres, n, w, form, test) {
   
   ## if no intersection or only 1 shared period of e_it and e_jt
   ## => exclude from calculation and issue a warning.
-  ## In general, length(m.ij) gives the number of shared periods by indiviudals i, j
+  ## In general, length(m.ij) gives the number of shared periods by individuals i, j
   ## Thus, non intersecting pairs are indicated by length(m.ij) == 0 (t.ij[i,j] == 0)
   no.one.intersect <- (t.ij <= 1)
   if (any(no.one.intersect, na.rm = TRUE)) {
@@ -383,9 +399,9 @@ pcdres <- function(tres, n, w, form, test) {
     number.of.non.one.intersecting.pairs <- sum(no.one.intersect, na.rm = TRUE)
     number.of.total.pairs <- (n*(n-1))/2
     share.on.one.intersect.pairs <- number.of.non.one.intersecting.pairs / number.of.total.pairs * 100
-    warning(paste("Some pairs of individuals (",
+    warning(paste0("Some pairs of individuals (",
                   signif(share.on.one.intersect.pairs, digits = 2),
-                  " percent) do not have any or just one time period in common and have been omitted from calculation", sep=""))
+                  " percent) do not have any or just one time period in common and have been omitted from calculation"))
     selector.mat[no.one.intersect] <- FALSE
   }
 
@@ -418,14 +434,17 @@ pcdres <- function(tres, n, w, form, test) {
     testname      <- "Scaled LM test"
    },
    bcsclm = {
-      CDstat        <- sqrt(1/(2*elem.num))*sum((t.ij*rho^2-1)[selector.mat]) - (n/(2*(max(t.ij)-1))) # Baltagi/Feng/Kao (2012), formula (11) (unbalanced case as sclm + in bias correction as EViews: max(T_ij) instead of T)
+     # Baltagi/Feng/Kao (2012), formula (11)
+     # (unbalanced case as sclm + bias correction as EViews: max(T_ij) instead of T)
+      CDstat        <- sqrt(1/(2*elem.num))*sum((t.ij*rho^2-1)[selector.mat]) - (n/(2*(max(t.ij)-1)))
       pCD           <- 2*pnorm(abs(CDstat), lower.tail = FALSE)
       names(CDstat) <- "z"
       parm          <- NULL
       testname      <- "Bias-corrected Scaled LM test"
    },
    cd = {
-    CDstat        <- sqrt(1/elem.num)*sum((sqrt(t.ij)*rho)[selector.mat]) # (Pesaran (2004), formula (31))
+     # (Pesaran (2004), formula (31))
+    CDstat        <- sqrt(1/elem.num)*sum((sqrt(t.ij)*rho)[selector.mat]) 
     pCD           <- 2*pnorm(abs(CDstat), lower.tail = FALSE)
     names(CDstat) <- "z"
     parm          <- NULL
@@ -463,19 +482,20 @@ preshape <- function(x, na.rm = TRUE, ...) {
     ## e.g., of residuals from a panelmodel,
     ## in wide form
     inames <- names(attr(x, "index"))
-    mres <- reshape(cbind(as.vector(x), attr(x, "index")),
+    mres <- reshape(cbind(as.vector(x),
+                          attr(x, "index")),
                     direction = "wide",
-                    timevar = inames[2],
-                    idvar = inames[1])
+                    timevar   = inames[2L],
+                    idvar     = inames[1L])
     ## drop ind in first column
-    mres <- mres[ , -1, drop = FALSE]
+    mres <- mres[ , -1L, drop = FALSE]
     ## reorder columns (may be scrambled depending on first
     ## available obs in unbalanced panels)
-    mres <- mres[ , order(dimnames(mres)[[2]])]
+    mres <- mres[ , order(dimnames(mres)[[2L]])]
     ## if requested, drop columns (time periods) with NAs
     if(na.rm) {
-        na.cols <- vapply(mres, FUN = anyNA, FUN.VALUE = TRUE)
-        if(sum(na.cols) > 0) mres <- mres[, !na.cols]
+        na.cols <- vapply(mres, FUN = anyNA, FUN.VALUE = TRUE, USE.NAMES = FALSE)
+        if(sum(na.cols) > 0L) mres <- mres[ , !na.cols, drop = FALSE]
     }
     return(mres)
 }
@@ -491,14 +511,23 @@ preshape <- function(x, na.rm = TRUE, ...) {
 #' @param x an object of class `pseries`
 #' @param grouping grouping variable,
 #' @param groupnames a character vector of group names,
-#' @param value to complete
-#' @param \dots further arguments
-#' @return A matrix
+#' @param value to complete,
+#' @param \dots further arguments.
+#' @return A matrix with average correlation coefficients within a group
+#' (diagonal) and between groups (off-diagonal).
 #' @export
 #' @keywords htest
+#' @examples
+#' 
+#' data("Grunfeld", package = "plm")
+#' pGrunfeld <- pdata.frame(Grunfeld)
+#' grp <- c(rep(1, 100), rep(2, 50), rep(3, 50)) # make 3 groups
+#' cortab(pGrunfeld$value, grouping = grp, groupnames = c("A", "B", "C"))
+#' 
 cortab <- function(x, grouping, groupnames = NULL,
                    value = "statistic", ...) {
-    ## makes table of within and between correlation
+    ## makes matrix containing within (diagonal) and between (off-diagonal)
+    ## correlation
     ## needs a pseries and a groupings vector of **same length**
 
     ## would use a better naming, and also passing a char or factor as
@@ -506,12 +535,12 @@ cortab <- function(x, grouping, groupnames = NULL,
 
     ## x must be a pseries
     if(!inherits(x, "pseries")) stop("First argument must be a pseries")
-    if(length(x) != length(grouping)) stop("Incompatible lengths")
+    if(length(x) != length(grouping)) stop("arguments 'x' and 'grouping' must have same length")
 
-    fullind <- as.numeric(attr(x, "index")[,1])
+    fullind <- as.numeric(attr(x, "index")[ , 1L])
     ids <- unique(fullind)
     n <- length(ids)
-    regs <- 1:length(unique(grouping))
+    regs <- seq_along(unique(grouping))
 
     if(!(is.numeric(grouping))) grouping <- as.numeric(as.factor(grouping))
     
@@ -525,17 +554,17 @@ cortab <- function(x, grouping, groupnames = NULL,
     ## for each pair of regions (nb: no duplicates, e.g., 3.1 but not 1.3)
 
     ## make w<1.n>:
-    for(h in 1:length(regs)) {
-      for(k in 1:h) {
-        statew <- matrix(0, ncol=n, nrow=n)
+    for(h in seq_along(regs)) {
+      for(k in seq_len(h)) {
+        statew <- matrix(0, ncol = n, nrow = n)
         ## make statew for cor. between h and k
-        for(i in 1:n) {
+        for(i in seq_len(n)) {
           ## get first region (all values equal, so take first one)
-          ireg <- grouping[fullind==ids[i]][1]
-          if(ireg==h) {
-            for(j in 1:n) {
-                jreg <- grouping[fullind==ids[j]][1]
-                if(jreg==k) statew[i,j] <- 1
+          ireg <- grouping[fullind == ids[i]][1L] # TODO: can be made faster via split()-approach
+          if(ireg == h) {
+            for(j in seq_len(n)) {
+                jreg <- grouping[fullind == ids[j]][1L] # TODO: can be made faster via split()-approach
+                if(jreg == k) statew[i, j] <- 1
             }
           }
         }
@@ -545,7 +574,7 @@ cortab <- function(x, grouping, groupnames = NULL,
         ## eliminate self.correlation of states if i=j
         diag(statew) <- 0
         ## not needed: pcdtest seems to do this by construction
-        eval(parse(text=paste("w", h, ".", k, " <- statew", sep="")))
+        eval(parse(text=paste0("w", h, ".", k, " <- statew")))
       }
      }
 
@@ -560,21 +589,18 @@ cortab <- function(x, grouping, groupnames = NULL,
     tab.g <- function(x, regs, regnames, test="rho", value) {
         myw <- 0
          tabg <- matrix(NA, ncol=length(regs), nrow=length(regs))
-         for(i in 1:length(regs)) {
-             for(j in 1:i) {
+         for(i in seq_along(regs)) {
+             for(j in seq_len(i)) {
                  ## take appropriate w matrix
-                 eval(parse(text=paste("myw<-w", i, ".", j, sep="")))
-                 tabg[i,j] <- pcdtest(x, test="rho", w=myw)[[value]]
+                 eval(parse(text = paste0("myw<-w", i, ".", j)))
+                 tabg[i, j] <- pcdtest(x, test = "rho", w = myw)[[value]]
              }
          }
          dimnames(tabg) <- list(groupnames, groupnames)
          return(tabg)
     }
     regnames <- ""
-    mytab <- tab.g(x, regs=regs, regnames=regnames, test="rho", value=value)
+    mytab <- tab.g(x, regs = regs, regnames = regnames, test = "rho", value = value)
     return(mytab)
 }
 
-
-
-
