@@ -27,7 +27,7 @@
 #' `coeftest()`, argument `vcov` to `waldtest()` and other methods in the
 #' \CRANpkg{lmtest} package; and argument `vcov.` to
 #' `linearHypothesis()` in the \CRANpkg{car} package (see the
-#' examples), see \insertCite{@ZEIL:04, 4.1-2 and examples below}{plm}.
+#' examples), \insertCite{@see also @ZEIL:04}{plm}, 4.1-2, and examples below.
 #' 
 #' @aliases vcovSCC
 #' @param x an object of class `"plm"` or `"pcce"`
@@ -114,7 +114,7 @@ vcovSCC <- function(x, ...){
 #' `coeftest()`, argument `vcov` to `waldtest()` and other methods in the
 #' \CRANpkg{lmtest} package; and argument `vcov.` to
 #' `linearHypothesis()` in the \CRANpkg{car} package (see the
-#' examples), see \insertCite{@ZEIL:04, 4.1-2 and examples below}{plm}.
+#' examples), see \insertCite{@see also @ZEIL:04}{plm}, 4.1-2, and examples below.
 #' 
 #' @aliases vcovNW
 #' @param x an object of class `"plm"` or `"pcce"`
@@ -198,7 +198,7 @@ vcovNW <- function(x, ...){
 #' `coeftest()`, argument `vcov` to `waldtest()` and other methods in the
 #' \CRANpkg{lmtest} package; and argument `vcov.` to
 #' `linearHypothesis()` in the \CRANpkg{car} package (see the
-#' examples), see \insertCite{@ZEIL:04, 4.1-2 and examples below}{plm}. 
+#' examples), see \insertCite{@see also @ZEIL:04}{plm}, 4.1-2, and examples below.
 #' 
 #' @aliases vcovDC
 #' @param x an object of class `"plm"` or `"pcce"`
@@ -266,13 +266,12 @@ vcovDC <- function(x, ...){
 #' @aliases vcovG
 #' @param x an object of class `"plm"` or `"pcce"`
 #' @param type the weighting scheme used, one of `"HC0"`,
-#'     `"sss"`, `"HC1"`, `"HC2"`, `"HC3"`,
-#'     `"HC4"`,
+#'     `"sss"`, `"HC1"`, `"HC2"`, `"HC3"`, `"HC4"`,
 #' @param cluster one of `"group"`, `"time"`,
 #' @param l lagging order, defaulting to zero
 #' @param inner the function to be applied to the residuals inside the
 #'     sandwich: one of `"cluster"` or `"white"` or
-#'     `"diagavg"`,
+#'     `"diagavg"`, or a user specified R function,
 #' @param \dots further arguments
 #' @return An object of class `"matrix"` containing the estimate
 #'     of the covariance matrix of coefficients.
@@ -280,8 +279,7 @@ vcovDC <- function(x, ...){
 #' @author Giovanni Millo
 #' @seealso [vcovHC()], [vcovSCC()],
 #'     [vcovDC()], [vcovNW()], and
-#'     [vcovBK()] albeit the latter does not make use of
-#'     vcovG.
+#'     [vcovBK()] albeit the latter does not make use of vcovG.
 #' @references
 #'
 #' \insertRef{mil17b}{plm}
@@ -331,7 +329,6 @@ vcovG.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
     }
 
   ## extract demeaned data
-    demy <- pmodel.response(x, model = model)
     demX <- model.matrix(x, model = model, rhs = 1, cstcovar.rm = "all")
     ## drop any linear dependent columns (corresponding to aliased coefficients)
     ## from model matrix X
@@ -354,52 +351,24 @@ vcovG.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
     nT <- pdim$nT$N
     Ti <- pdim$Tint$Ti
     k <- dim(demX)[[2L]]
-    n0 <- pdim$nT$n
-    t0 <- pdim$nT$T
+    n0 <- pdim$nT$n # no. of groups
+    t0 <- pdim$nT$T # max time periods over groups
 
   ## extract residuals
     uhat <- x$residuals
 
-  ## define residuals weighting function omega(res)
-  ## (code taken from meatHC and modified)
-  ## (the weighting is defined "in sqrt" relative to the literature)
-  ## 
-  ## (see the theoretical comments in pvcovHC)
-
-    ## this is computationally heavy, do only if needed
-    switch(match.arg(type), "HC0" = {diaghat <- NULL},
-                            "sss" = {diaghat <- NULL},
-                            "HC1" = {diaghat <- NULL},
-                            "HC2" = {diaghat <- try(dhat(demX), silent = TRUE)},
-                            "HC3" = {diaghat <- try(dhat(demX), silent = TRUE)},
-                            "HC4" = {diaghat <- try(dhat(demX), silent = TRUE)})
-    df <- nT - k
-    switch(match.arg(type), 
-           "HC0" = {
-            omega <- function(residuals, diaghat, df, g) residuals
-        }, "sss" = {
-            omega <- function(residuals, diaghat, df, g) residuals *
-                                sqrt(g/(g-1)*((nT-1)/(nT-k)))
-        }, "HC1" = {
-            omega <- function(residuals, diaghat, df, g) residuals *
-                                sqrt(length(residuals)/df)
-        }, "HC2" = {
-            omega <- function(residuals, diaghat, df, g) residuals /
-                                sqrt(1 - diaghat)
-        }, "HC3" = {
-            omega <- function(residuals, diaghat, df, g) residuals /
-                                (1 - diaghat)
-        }, "HC4" = {
-            omega <- function(residuals, diaghat, df, g) {
-                residuals/sqrt(1 - diaghat)^
-                 pmin(4, length(residuals) *
-                      diaghat/as.integer(round(sum(diaghat),
-                digits = 0)))
-            }
-        })
+    diaghat <- switch(type, "HC0" = NULL,
+                            "sss" = NULL,
+                            "HC1" = NULL,
+                            "HC2" = try(dhat(demX), silent = TRUE),
+                            "HC3" = try(dhat(demX), silent = TRUE),
+                            "HC4" = try(dhat(demX), silent = TRUE))
+    
 
    ## Definition module for E(u,v)
     if(is.function(inner)) {
+      # case of user-specified function
+      # (only if user calls workhorse vcovG directly (i.e., not possible via wrappers vcovXX)
         E <- inner
     } else {
       ## outer for clustering/arellano, diag(diag(inner)) for white
@@ -425,8 +394,8 @@ vcovG.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
                     eres <- array(0, dim = dim(efull))
                     dimnames(eres) <- dimnames(efull)
                     ## populate "pseudo-diagonal" with values from efull
-                    for(i in 1:length(names(u))) {
-                        for(j in 1:length(names(v))) {
+                    for(i in seq_along(names(u))) {
+                        for(j in seq_along(names(v))) {
                             if(names(u)[i] == names(v)[j]) {
                                 eres[i, j] <- efull[i, j]
                             }
@@ -452,8 +421,8 @@ vcovG.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
                     eres <- array(0, dim = dim(efull))
                     dimnames(eres) <- dimnames(efull)
                     ## populate "pseudo-diagonal" with values from efull
-                    for(i in 1:length(names(u))) {
-                        for(j in 1:length(names(v))) {
+                    for(i in seq_along(names(u))) {
+                        for(j in seq_along(names(v))) {
                             if(names(u)[i] == names(v)[j]) {
                                 eres[i, j] <- efull[i, j]
                             }
@@ -481,22 +450,43 @@ vcovG.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
     groupind <- as.numeric(xindex[[1L]])
     timeind  <- as.numeric(xindex[[2L]])
 
+
   ## adjust for 'fd' model (losing first time period)
     if(model == "fd") {
+      ## debug printing:
+      #print("before FD adj:")
+      #print(paste0("nT = ", nT))
+      #print(paste0("Ti = ", paste0(Ti, collapse = ", ")))
+      #print(paste0("t0 = ", t0))
+      #cat("\n")
+   
       groupi <- as.numeric(groupind)
       ## make vector =1 on first obs in each group, 0 elsewhere
       selector <- groupi - c(0, groupi[-length(groupi)])
       selector[1L] <- 1 # the first must always be 1
       ## eliminate first obs in time for each group
       groupind <- groupind[!selector]
-      timeind <- timeind[!selector]
+      timeind  <- timeind[!selector]
       nT <- nT - n0
       Ti <- Ti - 1
+      if(any(drop <- Ti == 0L)) {
+        # drop groups in Ti that are now empty (group had 1 observation before first-differencing, hence 0 after)
+        # and adjust n0 due to same reason
+        Ti <- Ti[!drop]
+        n0 <- n0 - sum(drop)
+      }
       t0 <- t0 - 1
+      
+      ## debug printing:
+      #print("after FD adj:")
+      #print(paste0("nT = ", nT))
+      #print(paste0("Ti = ", paste0(Ti, collapse = ", ")))
+      #print(paste0("t0 = ", t0))
     }
 
   ## set grouping indexes
-    switch(match.arg(cluster),
+    cluster <- match.arg(cluster)
+    switch(cluster,
             "group" = {
               n <- n0
               t <- t0
@@ -508,28 +498,26 @@ vcovG.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
               relevant.ind <- timeind
               lab <- groupind})
     
-    tind <- vector("list", n)
-    tlab <- vector("list", n)
-    
-    for (i in 1:length(unique(relevant.ind))) {
-        tind[[i]] <- which(relevant.ind == i)
-        tlab[[i]] <- lab[which(relevant.ind == i)]
-    }
-  
+    relevant.ind.GRP <- collapse::GRP(relevant.ind)
+    tind <- collapse::gsplit(seq_along(relevant.ind), relevant.ind.GRP)
+    tlab <- collapse::gsplit(lab, relevant.ind.GRP)
+
   ## lab were the 'labels' (a numeric, actually) for the relevant index;
   ## in use again from the need to make pseudo-diagonals for
   ## calc. the lagged White terms on unbalanced panels
 
   ## transform residuals by weights (here because type='sss' needs to
-  ## know who the grouping index 'g' is
+  ## know who the grouping index 'g' is in helper function omega())
 
   ## set number of clusters for Stata-like small sample correction
   ## (if clustering, i.e., inner="cluster", then G is the cardinality of
   ## the grouping index; if inner="white" it is simply the sample size)
-    ## find some more elegant solution for this!
+    ## TODO: find some more elegant solution for this!
     ## (perhaps if white then sss -> HC1 but check...)
   G <- if(match.arg(inner) == "cluster") n else nT
-  uhat <- omega(uhat, diaghat, df, G)
+  
+  # transform residuals by weights
+  uhat <- omega(residuals = uhat, diaghat = diaghat, g = G, nT = nT, k = k, type = type)
 
   ## compute basic block: X'_t u_t u'_(t-l) X_(t-l) foreach t,
   ## then calculate Sl_t and sum over t (here i in place of t)
@@ -539,19 +527,20 @@ vcovG.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
 
     ## preallocate k x k x (T-l) array for 'pile' of kxk matrices
     ## holding the X' E(u,ul) X elements
-    Sl <- array(dim = c(k, k, n-l))
+    Sl <- array(NA_real_, dim = c(k, k, n-l))
     
     ## (l=0 gives the special contemporaneous case where Xi=Xil, ui=uil
     ## for computing W, CX, CT)
+
     for(i in (1+l):n) {
       X  <- demX[tind[[i]], ,   drop = FALSE]
       Xl <- demX[tind[[i-l]], , drop = FALSE]
       u  <- uhat[tind[[i]]]
       ul <- uhat[tind[[(i-l)]]]
-      names(u)  <- tlab[[i]]
-      names(ul) <- tlab[[(i-l)]]
+      names(u)  <- tlab[[i]]      # names needed in E()
+      names(ul) <- tlab[[(i-l)]]  #   --- " ---
       ## calculate V_yy
-      Sl[ , , i-l] <- crossprod(X, E(u, ul)) %*% Xl
+      Sl[ , , i-l] <- tcrossprod(crossprod(X, E(u, ul)), t(Xl))
     }
     
     ## in order to sum on available observations two things can be done:
@@ -573,11 +562,11 @@ vcovG.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
     ## bread by standard method
     pane <- solve(crossprod(demX))
     ## sandwich
-    mycov <-  tcrossprod(crossprod(t(pane), salame), t(pane)) # == pane %*% salame %*% pane
+    mycov <- tcrossprod(tcrossprod(pane, salame), t(pane)) # == pane %*% salame %*% pane
     
     # save information about cluster variable in matrix (needed for e.g.,
     # robust F test)
-    attr(mycov, which = "cluster") <- match.arg(cluster)
+    attr(mycov, which = "cluster") <- cluster
     return(mycov)
 }
 
@@ -625,10 +614,11 @@ vcovG.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
 #' `coeftest()`, argument `vcov` to `waldtest()` and other methods in the
 #' \CRANpkg{lmtest} package; and argument `vcov.` to
 #' `linearHypothesis()` in the \CRANpkg{car} package (see the
-#' examples), see \insertCite{@ZEIL:04, 4.1-2 and examples below}{plm}.
+#' examples), see \insertCite{@see also @ZEIL:04}{plm}, 4.1-2, and examples below.
 #' 
-#' A special procedure for `pgmm` objects, proposed by
-#' \insertCite{WIND:05;textual}{plm}, is also provided.
+#' A method for `pgmm` objects, `vcovHC.pgmm`, is also provided and gives the robust
+#' variance-covariances matrix, in case of a two-steps panel GMM model with the 
+#' small-sample correction proposed by \insertCite{WIND:05;textual}{plm}.
 #' 
 #' @name vcovHC.plm
 #' @aliases vcovHC
@@ -720,9 +710,9 @@ NULL
 
 #' @rdname vcovHC.plm
 #' @export
-vcovHC.plm <- function(x, method=c("arellano", "white1", "white2"),
-                       type=c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
-                       cluster=c("group", "time"), ...) {
+vcovHC.plm <- function(x, method = c("arellano", "white1", "white2"),
+                       type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
+                       cluster = c("group", "time"), ...) {
     ## user-level wrapper for White-Arellano covariances
 
     ## translate arguments
@@ -731,35 +721,32 @@ vcovHC.plm <- function(x, method=c("arellano", "white1", "white2"),
                     "white1"   = "white",
                     "white2"   = "diagavg")
 
-    return(vcovG(x, type=type, cluster=cluster,
-                        l=0, inner=inner, ...))
+    return(vcovG(x, type = type, cluster = cluster, l = 0, inner = inner, ...))
 }
 
 #' @rdname vcovNW
 #' @export
-vcovNW.plm <- function(x, type=c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
-                       maxlag=NULL,
-                       wj=function(j, maxlag) 1-j/(maxlag+1),
+vcovNW.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
+                       maxlag = NULL,
+                       wj = function(j, maxlag) 1-j/(maxlag+1),
                        ...) {
     ## user-level wrapper for panel Newey-West estimator
 
     ## set default lag order
     if(is.null(maxlag)) maxlag <- floor((max(pdim(x)$Tint$Ti))^(1/4))
 
-    return(vcovSCC(x, type=type, maxlag=maxlag, inner="white", wj=wj, ...))
+    return(vcovSCC(x, type = type, maxlag = maxlag, inner = "white", wj = wj, ...))
 }
 
 #' @rdname vcovDC
 #' @export
-vcovDC.plm <- function(x, type=c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
+vcovDC.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
                        ...) {
     ## user-level wrapper for double-clustering (no persistence)
 
-    Vcx <- vcovG(x, type=type, cluster="group",
-                        l=0, inner="cluster", ...)
-    Vct <- vcovG(x, type=type, cluster="time",
-                        l=0, inner="cluster", ...)
-    Vw <- vcovG(x, type=type, l=0, inner="white", ...)
+    Vcx <- vcovG(x, type = type, cluster = "group", l = 0, inner = "cluster", ...)
+    Vct <- vcovG(x, type = type, cluster = "time",  l = 0, inner = "cluster", ...)
+    Vw  <- vcovG(x, type = type,                    l = 0, inner = "white", ...)
 
     res <- Vcx + Vct - Vw
     
@@ -771,15 +758,13 @@ vcovDC.plm <- function(x, type=c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
 
 #' @rdname vcovSCC
 #' @export
-vcovSCC.plm <- function(x, type=c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
-                        cluster="time",
-                        maxlag=NULL,
-                        inner=c("cluster", "white", "diagavg"),
-                        wj=function(j, maxlag) 1-j/(maxlag+1),
+vcovSCC.plm <- function(x, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
+                        cluster = "time",
+                        maxlag = NULL,
+                        inner = c("cluster", "white", "diagavg"),
+                        wj = function(j, maxlag) 1-j/(maxlag+1),
                         ...) {
 
-    ## replicates vcovSCC
-
     ## set default lag order
     if(is.null(maxlag)) maxlag <- floor((max(pdim(x)$Tint$Ti))^(1/4))
 
@@ -787,12 +772,11 @@ vcovSCC.plm <- function(x, type=c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
     ## wj <- function(j, maxlag) 1-j/(maxlag+1)
     ## has been passed as argument
 
-    S0 <- vcovG(x, type=type, cluster=cluster, l=0, inner=inner)
+    S0 <- vcovG(x, type = type, cluster = cluster, l = 0, inner = inner)
 
     if(maxlag > 0) {
-        for(i in 1:maxlag) {
-            Vctl <- vcovG(x, type=type, cluster=cluster,
-                             l=i, inner=inner)
+        for(i in seq_len(maxlag)) {
+            Vctl <- vcovG(x, type = type, cluster = cluster, l = i, inner = inner)
             S0 <- S0 + wj(i, maxlag) * (Vctl + t(Vctl))
         }
     }
@@ -805,8 +789,6 @@ vcovSCC.plm <- function(x, type=c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
 
 ## separate function for BK (PCSE) covariance
 
-
-
 #' Beck and Katz Robust Covariance Matrix Estimators
 #' 
 #' Unconditional Robust covariance matrix estimators *a la Beck
@@ -827,7 +809,7 @@ vcovSCC.plm <- function(x, type=c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
 #' elsewhere.
 #' 
 #' The `diagonal` logical argument can be used, if set to
-#' `TRUE`, to force to zero all nondiagonal elements in the
+#' `TRUE`, to force to zero all non-diagonal elements in the
 #' estimated error covariances; this is appropriate if both serial and
 #' cross--sectional correlation are assumed out, and yields a
 #' timewise- (groupwise-) heteroskedasticity--consistent estimator.
@@ -851,14 +833,14 @@ vcovSCC.plm <- function(x, type=c("HC0", "sss", "HC1", "HC2", "HC3", "HC4"),
 #' `coeftest()`, argument `vcov` to `waldtest()` and other methods in the
 #' \CRANpkg{lmtest} package; and argument `vcov.` to
 #' `linearHypothesis()` in the \CRANpkg{car} package (see the
-#' examples), see \insertCite{@ZEIL:04, 4.1-2 and examples below}{plm}.
+#' examples), see \insertCite{@see also @ZEIL:04}{plm}, 4.1-2, and examples below.
 #' 
 #' @param x an object of class `"plm"`,
-#' @param type the weighting scheme used, one of `"HC0"`, `"HC1"`,
+#' @param type the weighting scheme used, one of `"HC0"`, `"sss"`, `"HC1"`,
 #'     `"HC2"`, `"HC3"`, `"HC4"`, see Details,
 #' @param cluster one of `"group"`, `"time"`,
 #' @param diagonal a logical value specifying whether to force
-#'     nondiagonal elements to zero,
+#'     non-diagonal elements to zero,
 #' @param \dots further arguments.
 #' @export
 #' @return An object of class `"matrix"` containing the estimate of
@@ -912,12 +894,9 @@ vcovBK <- function(x, ...) {
     UseMethod("vcovBK")
 }
 
-
-# TODO: add type "sss" for vcovBK
-
 #' @rdname vcovBK
 #' @export
-vcovBK.plm <- function(x, type = c("HC0", "HC1", "HC2", "HC3", "HC4"),
+vcovBK.plm <- function(x, type = c("HC0", "sss" , "HC1", "HC2", "HC3", "HC4"),
                        cluster = c("group", "time"),
                        diagonal = FALSE, ...) {
 
@@ -933,13 +912,13 @@ vcovBK.plm <- function(x, type = c("HC0", "HC1", "HC2", "HC3", "HC4"),
   ## this average block (say, OmegaM in EViews notation) is then put into
   ## White's formula instead of each Omega_i.
   ##
-  ## The clustering defaults to "group" for consistency with pvcovHC;
+  ## The clustering defaults to "group" for consistency with vcovHC;
   ## nevertheless the most likely usage is cluster="time" for robustness vs.
   ## cross-sectional dependence, as in the original Beck and Katz paper (where
   ## it is applied to "pooling" models).
   ##
   ## This version: compliant with plm 1.2-0; lmtest.
-  ## Code is identical to pvcovHC until mark.
+  ## Code is identical to vcovHC until mark.
   ##
   ## Usage:
   ## myplm <- plm(<model>,<data>, ...)
@@ -968,7 +947,6 @@ vcovBK.plm <- function(x, type = c("HC0", "HC1", "HC2", "HC3", "HC4"),
     }
     
   ## extract demeaned data
-    demy <- pmodel.response(x, model = model)
     demX <- model.matrix(x, model = model, rhs = 1, cstcovar.rm = "all")
     ## drop any linear dependent columns (corresponding to aliased coefficients)
     ## from model matrix X
@@ -1007,17 +985,42 @@ vcovBK.plm <- function(x, type = c("HC0", "HC1", "HC2", "HC3", "HC4"),
     groupind <- as.numeric(xindex[[1L]])
     timeind  <- as.numeric(xindex[[2L]])
 
-  ## Achim's fix for 'fd' model (losing first time period)
+    ## adjust for 'fd' model (losing first time period) [same code as in vcovG.plm]
     if(model == "fd") {
-      groupind <- groupind[timeind > 1]
-      timeind <- timeind[timeind > 1]
+      ## debug printing:
+      #print("before FD adj:")
+      #print(paste0("nT = ", nT))
+      #print(paste0("Ti = ", paste0(Ti, collapse = ", ")))
+      #print(paste0("t0 = ", t0))
+      #cat("\n")
+      
+      groupi <- as.numeric(groupind)
+      ## make vector =1 on first obs in each group, 0 elsewhere
+      selector <- groupi - c(0, groupi[-length(groupi)])
+      selector[1L] <- 1 # the first must always be 1
+      ## eliminate first obs in time for each group
+      groupind <- groupind[!selector]
+      timeind  <- timeind[!selector]
       nT <- nT - n0
       Ti <- Ti - 1
+      if(any(drop <- Ti == 0L)) {
+        # drop groups in Ti that are now empty (group had 1 observation before first-differencing, hence 0 after)
+        # and adjust n0 due to same reason
+        Ti <- Ti[!drop]
+        n0 <- n0 - sum(drop)
+      }
       t0 <- t0 - 1
+      
+      ## debug printing:
+      #print("after FD adj:")
+      #print(paste0("nT = ", nT))
+      #print(paste0("Ti = ", paste0(Ti, collapse = ", ")))
+      #print(paste0("t0 = ", t0))
     }
-
+    
   ## set grouping indexes
-    switch(match.arg(cluster),
+    cluster <- match.arg(cluster)
+    switch(cluster,
             "group" = {
               n <- n0 # this is needed only for 'pcse'
               t <- t0 # this is needed only for 'pcse'
@@ -1030,49 +1033,20 @@ vcovBK.plm <- function(x, type = c("HC0", "HC1", "HC2", "HC3", "HC4"),
               lab <- groupind
             })
     
-    tind <- vector("list", n)
-    tlab <- vector("list", n)
+    relevant.ind.GRP <- collapse::GRP(relevant.ind)
+    tind <- collapse::gsplit(seq_along(relevant.ind), relevant.ind.GRP)
+    tlab <- collapse::gsplit(lab, relevant.ind.GRP)
     
-    for (i in 1:length(unique(relevant.ind))) {
-        tind[[i]] <- which(relevant.ind == i)
-        tlab[[i]] <- lab[which(relevant.ind == i)]
-    }
-
-  ## define residuals weighting function omega(res)
-  ## (code taken from meatHC and modified)
-  ## (the weighting is defined "in sqrt" relative to the literature)
-  ##
-  ## (see the theoretical comments in pvcovHC)
-
-    ## this is computationally heavy, do only if needed
-    switch(match.arg(type), "HC0" = {diaghat <- NULL},
-                            "HC1" = {diaghat <- NULL},
-                            "HC2" = {diaghat <- try(dhat(demX), silent = TRUE)},
-                            "HC3" = {diaghat <- try(dhat(demX), silent = TRUE)},
-                            "HC4" = {diaghat <- try(dhat(demX), silent = TRUE)})
-    df <- nT - k
-    switch(match.arg(type), 
-           "HC0" = {
-            omega <- function(residuals, diaghat, df) residuals
-        }, "HC1" = {
-            omega <- function(residuals, diaghat, df) residuals *
-                                sqrt(length(residuals)/df)
-        }, "HC2" = {
-            omega <- function(residuals, diaghat, df) residuals /
-                                sqrt(1 - diaghat)
-        }, "HC3" = {
-            omega <- function(residuals, diaghat, df) residuals /
-                                (1 - diaghat)
-        }, "HC4" = {
-            omega <- function(residuals, diaghat, df) residuals/sqrt(1 -
-                diaghat)^pmin(4, length(residuals) * diaghat/as.integer(round(sum(diaghat),
-                digits = 0)))
-        })
-
+    diaghat <- switch(type, "HC0" = NULL,
+                            "sss" = NULL,
+                            "HC1" = NULL,
+                            "HC2" = try(dhat(demX), silent = TRUE),
+                            "HC3" = try(dhat(demX), silent = TRUE),
+                            "HC4" = try(dhat(demX), silent = TRUE))
+  
   ## transform residuals by weights
-  uhat <- omega(uhat, diaghat, df)
-
-  ## CODE TAKEN FROM pvcovHC() UNTIL HERE except for ind/time labeling ##
+  uhat <- omega(residuals = uhat, diaghat = diaghat,
+                g = n, nT = nT, k = k, type = type)
 
   ## the PCSE covariance estimator is based on the unconditional estimate
   ## of the intragroup (intraperiod) covariance of errors, OmegaT or OmegaM
@@ -1087,18 +1061,22 @@ vcovBK.plm <- function(x, type = c("HC0", "HC1", "HC2", "HC3", "HC4"),
 
     ## est. omega submatrix
     ## "pre-allocate" an empty array
-    tres <- array(dim = c(t, t, n))
+    tres <- array(NA_real_, dim = c(t, t, n))
 
     ## array of n "empirical omega-blocks"
     ## with outer product of t(i) residuals
     ## for each group 1..n
     ## (use subscripting from condition 'label in labels' set',
     ## the rest stays NA if any)
-    for(i in 1:n) {
+    
+    unlabs <- unique(lab) # fetch (all, unique) values of the relevant labels
+    seq.len.t <- seq_len(t)
+        
+    for(i in seq_len(n)) {
       ut <- uhat[tind[[i]]]
-      tpos <- (1:t)[unique(lab) %in% tlab[[i]]]
-      ## put nondiag elements to 0 if diagonal=TRUE
-      tres[tpos, tpos, i] <- if(diagonal) diag(diag(ut %o% ut)) else ut %o% ut
+      tpos <- seq.len.t[unlabs %in% tlab[[i]]]
+      ## put non-diag elements to 0 if diagonal=TRUE
+      tres[tpos, tpos, i] <- if(diagonal) diag(diag(tcrossprod(ut))) else tcrossprod(ut)
     }
 
     ## average over all omega blocks, removing NAs (apply preserving
@@ -1107,18 +1085,15 @@ vcovBK.plm <- function(x, type = c("HC0", "HC1", "HC2", "HC3", "HC4"),
     OmegaT <- rowMeans(tres, dims = 2L, na.rm = TRUE) # == apply(tres, 1:2, mean, na.rm = TRUE) but faster
   ## end of PCSE covariance calculation.
 
-  ## fetch (all, unique) values of the relevant labels
-  unlabs <- unique(lab)
-
-  salame <- array(dim = c(k, k, n))
-  for(i in 1:n) {
+  salame <- array(NA_real_, dim = c(k, k, n))
+  for(i in seq_len(n)) {
     groupinds <- tind[[i]]
     grouplabs <- tlab[[i]]
     xi <- demX[groupinds, , drop = FALSE]
     ## for every group, take relevant positions
     tpos <- unlabs %in% grouplabs
     OmegaTi <- OmegaT[tpos, tpos, drop = FALSE]
-    salame[ , , i] <- crossprod(xi, OmegaTi) %*% xi
+    salame[ , , i] <- tcrossprod(crossprod(xi, OmegaTi), t(xi))
   }
   ## meat
   salame <- rowSums(salame, dims = 2L) # == apply(salame, 1:2, sum) but faster
@@ -1127,11 +1102,11 @@ vcovBK.plm <- function(x, type = c("HC0", "HC1", "HC2", "HC3", "HC4"),
   pane <- solve(crossprod(demX))
 
   ## sandwich
-  mycov <- tcrossprod(crossprod(t(pane), salame), t(pane)) # == pane %*% salame %*% pane
+  mycov <- tcrossprod(tcrossprod(pane, salame), t(pane)) # == pane %*% salame %*% pane
   
   # save information about cluster variable in matrix (needed for e.g.,
   # robust F test)
-  attr(mycov, which = "cluster") <- match.arg(cluster)
+  attr(mycov, which = "cluster") <- cluster
   return(mycov)
 }
 
@@ -1145,24 +1120,53 @@ vcovBK.plm <- function(x, type = c("HC0", "HC1", "HC2", "HC3", "HC4"),
 ## for any vcov that makes sense computed on the transformed
 ## data from model.matrix.pcce and pmodel.response.pcce
 
-## TODO: vcovBK.pcce missing? Or not valid?
+## TODO: vcovBK.pcce, vcovDC.pcce missing? Or not valid?
+##       Have a stopping break for now w/ informative error
+
+#' @export
+vcovBK.pcce <- function(x, ...) stop("plm::vcovBK not implemented for 'pcce' models")
+
+#' @export
+vcovDC.pcce <- function(x, ...) stop("plm::vcovDC not implemented for 'pcce' models")
 
 #' @rdname vcovG
 #' @export
-vcovG.pcce   <- vcovG.plm
+vcovG.pcce <- vcovG.plm
 
 #' @rdname vcovHC.plm
 #' @export
-vcovHC.pcce  <- vcovHC.plm
+vcovHC.pcce <- vcovHC.plm
 
 #' @rdname vcovNW
 #' @export
-vcovNW.pcce  <- vcovNW.plm
+vcovNW.pcce <- vcovNW.plm
 
 #' @rdname vcovSCC
 #' @export
 vcovSCC.pcce <- vcovSCC.plm
 
+########################################################
+## vcovXX methods for pggls objects as stopping break ##
+########################################################
+vcovXX.pggls.error <- "not implemented as not sensible for 'pggls' models"
+#' @export
+vcovG.pggls <- function(x, ...) stop(paste0("plm::vcovG ", vcovXX.pggls.error))
+
+#' @export
+vcovHC.pggls <- function(x, ...) stop(paste0("plm::vcovHC ", vcovXX.pggls.error))
+
+#' @export
+vcovNW.pggls <- function(x, ...) stop(paste0("plm::vcovNW ", vcovXX.pggls.error))
+
+#' @export
+vcovSCC.pggls <- function(x, ...) stop(paste0("plm::vcovSCC ", vcovXX.pggls.error))
+
+#' @export
+vcovBK.pggls <- function(x, ...) stop(paste0("plm::vcovBK ", vcovXX.pggls.error))
+
+#' @export
+vcovDC.pggls <- function(x, ...) stop(paste0("plm::vcovDC ", vcovXX.pggls.error))
+
 
 ####################################
 ## vcovHC method for pgmm objects ##
@@ -1176,6 +1180,7 @@ vcovHC.pgmm <- function(x, ...) {
   transformation <- describe(x, "transformation")
   A1 <- x$A1
   A2 <- x$A2
+  B1 <- x$B1 # needs to be B1 (from one-step model)
 
   if(transformation == "ld") {
 ##     yX <- lapply(x$model,function(x) rbind(diff(x),x))
@@ -1184,9 +1189,11 @@ vcovHC.pgmm <- function(x, ...) {
     residuals <- x$residuals
   }
   else {
+    # transformation = "d"
     yX <- x$model
     residuals <- x$residuals
   }
+  
   minevA2 <- min(abs(Re(eigen(A2)$values)))
   eps <- 1E-9
   
@@ -1195,49 +1202,57 @@ vcovHC.pgmm <- function(x, ...) {
     ginv(A2)
   } else solve(A2)
   
+  WX <- Reduce("+", mapply(function(w, y) crossprod(w, y[ , -1L, drop = FALSE]), x$W, yX, SIMPLIFY = FALSE))
+  
+  # robust vcov for one-step GMM, see Roodman (2009), formula (15)
+  vcovr1s <- B1 %*% (t(WX) %*% A1 %*% SA2 %*% A1 %*% WX) %*% B1
+  
   if(model == "twosteps") {
     coef1s <- x$coefficients[[1L]]
     res1s <- lapply(yX, function(x) x[ , 1L] - crossprod(t(x[ , -1L, drop = FALSE]), coef1s))
     K <- ncol(yX[[1L]])
     D <- c()
-    WX <- Reduce("+",
-                 mapply(function(x, y) crossprod(x, y[ , -1L, drop = FALSE]), x$W, yX, SIMPLIFY = FALSE))
     We <- Reduce("+", mapply(function(x, y) crossprod(x, y), x$W, residuals, SIMPLIFY = FALSE))
-    B1 <- solve(t(WX) %*% A1 %*% WX)
-    B2 <- vcov(x)
-
-    vcov1s <- B1 %*% (t(WX) %*% A1 %*% SA2 %*% A1 %*% WX) %*% B1
     for (k in 2:K) {
-      exk <- mapply(
-                    function(x, y){
+      exk <- mapply(function(x, y){
                       z <- crossprod(t(x[ , k, drop = FALSE]), t(y))
-                      - z - t(z)
+                      return(- z - t(z))
                     },
                     yX, res1s, SIMPLIFY = FALSE)
-      wexkw <- Reduce("+",
-                      mapply(
-                             function(x, y)
-                             crossprod(x, crossprod(y, x)),
-                             x$W, exk, SIMPLIFY = FALSE))
+      
+      wexkw <- Reduce("+", mapply(function(x, y) 
+                                    crossprod(x, crossprod(y, x)),
+                                  x$W, exk, SIMPLIFY = FALSE))
+      B2 <- x$vcov # is "B2" for a two-step model
       Dk <- -B2 %*% t(WX) %*% A2 %*% wexkw %*% A2 %*% We
       D <- cbind(D, Dk)
     }
-    vcovr <- B2 + crossprod(t(D), B2) + t(crossprod(t(D), B2)) + D %*% vcov1s %*% t(D)
+    # Windmeijer (2005) small-sample bias correction for twosteps GMM model, 
+    # see Windmeijer (2005), p. 33 formula (3.3); Roodman (2019) form. (18)
+    vcovr2s <- B2 + crossprod(t(D), B2) + t(crossprod(t(D), B2)) + D %*% vcovr1s %*% t(D)
   }
-  else {
-    # model = "onestep"
-    res1s <- lapply(yX, function(z) z[ , 1L] - crossprod(t(z[ , -1L, drop = FALSE]), x$coefficients))
-    K <- ncol(yX[[1L]])
-    WX <- Reduce("+", mapply(function(z, y) crossprod(z[ , -1L, drop = FALSE], y), yX, x$W, SIMPLIFY = FALSE))
-    B1 <- vcov(x)
-    vcovr <- B1 %*% (WX %*% A1 %*% SA2 %*% A1 %*% t(WX)) %*% B1
-  }
-  vcovr
+  if(model == "twosteps") vcovr2s else vcovr1s
 }
 
 
 ## dhat: diaghat function for matrices
-# old: dhat <- function(x) {tx <- t(x); diag(crossprod(tx, solve(crossprod(x), tx)))}
 dhat <- function(x) {
-  rowSums(crossprod(t(x), solve(crossprod(x))) * x) # == diag(crossprod(tx, solve(crossprod(x), tx)))
+  rowSums(crossprod(t(x), solve(crossprod(x))) * x) # == (old) diag(crossprod(t(x), solve(crossprod(x), t(x)))
+}
+
+## omega: weighting function, used in vcvG and vcovBK
+## define residuals weighting function omega(res)
+## (code taken from meatHC and modified)
+## (the weighting is defined "in sqrt" relative to the literature)
+omega <- function(residuals, diaghat, g, nT, k, type = c("HC0", "sss", "HC1", "HC2", "HC3", "HC4")) {
+  type <- match.arg(type)
+  switch(type,
+         "HC0" = { residuals },
+         "sss" = { residuals * sqrt(g/(g-1) * ((nT-1) / (nT-k))) },
+         "HC1" = { residuals * sqrt(length(residuals) / (nT-k))  },
+         "HC2" = { residuals / sqrt(1 - diaghat) },
+         "HC3" = { residuals /     (1 - diaghat) },
+         "HC4" = { residuals / sqrt(1 - diaghat)^pmin(4, length(residuals) *
+                                                        diaghat/as.integer(round(sum(diaghat), digits = 0))) }
+  )
 }
\ No newline at end of file
